@@ -4,10 +4,11 @@ use crate::{
     std_lib::{BinaryHeap, Ordering},
 };
 
-/// The prefix tree map builder.
+/// The prefix tree map builder
 #[derive(Clone)]
 pub struct PrefixTreeMapBuilder<E, W, V> {
     root: NodeBuilder<E, W, V>,
+    max_wildcard_depth: usize,
 }
 
 #[derive(Clone)]
@@ -22,7 +23,7 @@ where
     E: Clone + Ord,
     W: Clone + Ord,
 {
-    /// Create a new `PrefixTreeMapBuilder`.
+    /// Create a new `PrefixTreeMapBuilder`
     pub fn new() -> Self {
         Self {
             root: NodeBuilder {
@@ -30,63 +31,86 @@ where
                 value: None,
                 children: None,
             },
+            max_wildcard_depth: 0,
         }
     }
 
-    /// Insert a new value into the prefix tree map.
+    /// Insert a new value into the prefix tree map
     ///
-    /// Key parts need to be marked by [`KeyPart`](enum.KeyPart.html).
+    /// Key parts need to be marked by [`KeyPart`](enum.KeyPart.html)
     ///
-    /// Insert into a existed key path could overwrite the value in it.
+    /// Insert into a existed key path could overwrite the value in it
     pub fn insert(&mut self, key: impl IntoIterator<Item = KeyPart<E, W>>, value: V) {
-        unsafe {
-            let mut node: *mut NodeBuilder<E, W, V> = &mut self.root;
+        let mut node = &mut self.root as *mut NodeBuilder<E, W, V>;
+        let mut wildcard_depth = 0;
 
-            for key_part in key {
-                if (*node).children.is_none() {
-                    let mut children = BinaryHeap::new();
-                    children.push(NodeBuilder::new(key_part));
+        for key_part in key {
+            if key_part.is_wildcard() {
+                wildcard_depth += 1;
+            }
+
+            if unsafe { (*node).children.is_none() } {
+                let mut children = BinaryHeap::new();
+                children.push(NodeBuilder::new(key_part));
+
+                unsafe {
                     (*node).children = Some(children);
+                }
 
-                    let child = (*node).children.as_ref().unwrap().peek().unwrap();
+                let child = unsafe {
+                    (*node)
+                        .children
+                        .as_ref()
+                        .unwrap_unchecked()
+                        .peek()
+                        .unwrap_unchecked()
+                };
+
+                let child_const_ptr = child as *const NodeBuilder<E, W, V>;
+                node = child_const_ptr as *mut NodeBuilder<E, W, V>;
+            } else {
+                let children = unsafe { (*node).children.as_mut().unwrap_unchecked() };
+
+                if let Some(child) = children
+                    .iter()
+                    .find(|child| child.key_part.as_ref() == Some(&key_part))
+                {
                     let child_const_ptr = child as *const NodeBuilder<E, W, V>;
                     node = child_const_ptr as *mut NodeBuilder<E, W, V>;
                 } else {
-                    let children = (*node).children.as_mut().unwrap();
+                    let key_part_cloned = key_part.clone();
+                    children.push(NodeBuilder::new(key_part_cloned));
 
-                    if let Some(child) = children
-                        .iter()
-                        .find(|child| child.key_part.as_ref() == Some(&key_part))
-                    {
-                        let child_const_ptr = child as *const NodeBuilder<E, W, V>;
-                        node = child_const_ptr as *mut NodeBuilder<E, W, V>;
-                    } else {
-                        let key_part_cloned = key_part.clone();
-                        children.push(NodeBuilder::new(key_part_cloned));
-
-                        let child = children
+                    let child = unsafe {
+                        children
                             .iter()
                             .find(|child| child.key_part.as_ref() == Some(&key_part))
-                            .unwrap();
-                        let child_const_ptr = child as *const NodeBuilder<E, W, V>;
-                        node = child_const_ptr as *mut NodeBuilder<E, W, V>;
-                    }
+                            .unwrap_unchecked()
+                    };
+
+                    let child_const_ptr = child as *const NodeBuilder<E, W, V>;
+                    node = child_const_ptr as *mut NodeBuilder<E, W, V>;
                 }
             }
+        }
 
+        unsafe {
             (*node).value = Some(value);
         }
+
+        self.max_wildcard_depth = self.max_wildcard_depth.max(wildcard_depth);
     }
 
-    /// Insert a new value in an exact key path.
+    /// Insert a new value in an exact key path
     pub fn insert_exact(&mut self, key: impl IntoIterator<Item = E>, value: V) {
         self.insert(key.into_iter().map(KeyPart::Exact), value);
     }
 
-    /// Build the prefix tree map.
+    /// Build the prefix tree map
     pub fn build(self) -> PrefixTreeMap<E, W, V> {
         PrefixTreeMap {
             root: Self::node_builder_to_node(self.root),
+            max_wildcard_depth: self.max_wildcard_depth,
         }
     }
 
